@@ -692,7 +692,11 @@ class ReceiveClientAsync(client.ReceiveClient, AMQPClientAsync):
     :param timeout: A timeout in milliseconds. The receiver will shut down if no
      new messages are received after the specified timeout. If set to 0, the receiver
      will never timeout and will continue to listen. The default is 0.
+     Set `shutdown_after_timeout` to `False` if keeping the receiver open after timeout is needed.
     :type timeout: float
+    :param shutdown_after_timeout: Whether to automatically shutdown the receiver
+     if no new messages are received after the specified timeout. Default is `True`.
+    :type shutdown_after_timeout: bool
     :param auto_complete: Whether to automatically settle message received via callback
      or via iterator. If the message has not been explicitly settled after processing
      the message will be accepted. Alternatively, when used with batch receive, this setting
@@ -840,8 +844,13 @@ class ReceiveClientAsync(client.ReceiveClient, AMQPClientAsync):
             if self._timeout > 0:
                 timespan = now - self._last_activity_timestamp
                 if timespan >= self._timeout:
-                    _logger.info("Timeout reached, closing receiver.")
-                    self._shutdown = True
+                    self._timeout_reached = True
+                    if self._shutdown_after_timeout:
+                        _logger.info("Timeout reached, closing receiver.")
+                        self._shutdown = True
+                    else:
+                        self._last_activity_timestamp = None  # To reuse the receiver, reset the timestamp
+                        _logger.info("Timeout reached, keeping receiver open.")
         else:
             self._last_activity_timestamp = now
         self._was_message_received = False
@@ -865,16 +874,19 @@ class ReceiveClientAsync(client.ReceiveClient, AMQPClientAsync):
         self._streaming_receive = True
         await self.open_async()
         self._message_received_callback = on_message_received
+        self._timeout_reached = False
+        self._last_activity_timestamp = None
         receiving = True
         try:
-            while receiving:
+            while receiving and not self._timeout_reached:
                 receiving = await self.do_work_async()
+            receiving = False
         except:
             receiving = False
             raise
         finally:
             self._streaming_receive = False
-            if not receiving:
+            if not receiving and self._shutdown_after_timeout:
                 await self.close_async()
 
     async def receive_message_batch_async(self, max_batch_size=None, on_message_received=None, timeout=0):
@@ -917,8 +929,10 @@ class ReceiveClientAsync(client.ReceiveClient, AMQPClientAsync):
         if len(batch) >= max_batch_size:
             return batch
 
-        while receiving and not expired and len(batch) < max_batch_size:
-            while receiving and self._received_messages.qsize() < max_batch_size:
+        self._timeout_reached = False
+        self._last_activity_timestamp = None
+        while receiving and not expired and len(batch) < max_batch_size and not self._timeout_reached:
+            while receiving and self._received_messages.qsize() < max_batch_size and not self._timeout_reached:
                 if timeout and self._counter.get_current_ms() > timeout:
                     expired = True
                     break
@@ -992,6 +1006,7 @@ class AsyncMessageIter(collections.abc.AsyncIterator):  # pylint: disable=no-mem
     def __init__(self, rcv_client, auto_complete=True):
         self._client = rcv_client
         self._client.auto_complete = False
+        self._client._timeout_reached = False  # pylint: disable=protected-access
         self.receiving = True
         self.auto_complete = auto_complete
         self.current_message = None
@@ -1002,7 +1017,7 @@ class AsyncMessageIter(collections.abc.AsyncIterator):  # pylint: disable=no-mem
         if self.current_message and self.auto_complete:
             self.current_message.accept()
         try:
-            while self.receiving and self._client._received_messages.empty():
+            while self.receiving and self._client._received_messages.empty() and not self._client._timeout_reached:
                 self.receiving = await self._client.do_work_async()
             if not self._client._received_messages.empty():
                 message = self._client._received_messages.get()
@@ -1014,5 +1029,5 @@ class AsyncMessageIter(collections.abc.AsyncIterator):  # pylint: disable=no-mem
             self.receiving = False
             raise
         finally:
-            if not self.receiving:
+            if not self.receiving and self._client._shutdown_after_timeout:
                 await self._client.close_async()
