@@ -377,7 +377,7 @@ class Message(object):
             return True
         return False
 
-    def reject(self, condition=None, description=None):
+    def reject(self, condition=None, description=None, info=None):
         """Send a response disposition to the service to indicate that
         a received message has been rejected. If the client is running in PeekLock
         mode, the service will wait on this disposition. Otherwise it will
@@ -389,6 +389,8 @@ class Message(object):
         :type condition: bytes or str
         :param description: A description/reason to accompany the rejection.
         :type description: bytes or str
+        :param info: Information about the error condition.
+        :type info: dict
         :rtype: bool
         :raises: TypeError if the message is being sent rather than received.
         """
@@ -396,6 +398,7 @@ class Message(object):
             self._response = errors.MessageRejected(
                 condition=condition,
                 description=description,
+                info=info,
                 encoding=self._encoding)
             self._settler(self._response)
             self.state = constants.MessageState.ReceivedSettled
@@ -1067,13 +1070,13 @@ class MessageHeader(object):
     """
 
     def __init__(self, header=None):
-        self.delivery_count = None
+        self.delivery_count = 0
         self.time_to_live = None
         self.first_acquirer = None
         self.durable = None
         self.priority = None
         if header:
-            self.delivery_count = header.delivery_count
+            self.delivery_count = header.delivery_count or 0
             self.time_to_live = header.time_to_live
             self.first_acquirer = header.first_acquirer
             self.durable = header.durable
@@ -1094,8 +1097,7 @@ class MessageHeader(object):
         :rtype: uamqp.c_uamqp.cHeader
         """
         header = c_uamqp.create_header()
-        if self.delivery_count is not None:
-            header.delivery_count = self.delivery_count
+        header.delivery_count = self.delivery_count or 0
         if self.time_to_live is not None:
             header.time_to_live = self.time_to_live
         if self.first_acquirer is not None:
